#!/usr/local/bin/perl
#
# Connect.pm -- Library to remove jumps in a time series and connect the y values for a plot.
#               Control args are:
#                  y_jump  = threshold to declare a jump
#                  x_gap   = gap size in x after which one declares a jump and then connects using a slope
#                  n_slope = # of y values to use in computing slope before a jump
#

package Connect;

use strict;
use constant DEBUG => 0;

use constant Y_JUMP_DEFAULT  => 100;
use constant X_GAP_DEFAULT   =>  30;
use constant N_SLOPE_DEFAULT =>   5;


sub simple {
    my($n, $y, $y_name, $y_jump, $z) = @_;
    my($i, $j, $jump);

    $y_jump = Y_JUMP_DEFAULT if $y_jump eq "";
    my @i_jump = ();

    my $y_adj = 0;
    $$z[0] = $$y[0];
    for ($i = 1; $i < $n; $i++ ) {
        $jump = $$y[$i] - $$y[$i-1];

        if ( abs($jump) > $y_jump ) {
	    $j = $i + 1;
	    push @i_jump, $j;
            if ( DEBUG ) { print STDERR "Connect::simple: jump in $y_name of $jump at pt $j\n" }
            $y_adj += $jump;
        }
	$$z[$i] = $$y[$i] - $y_adj;
    }
    return @i_jump;
}


sub median {
  my($n, $x) = @_;

  my @xs = sort { $a <=> $b } @$x;
  my $nm = int($n / 2) + 1;
  return $xs[$nm];
}

1;
