#!/usr/local/bin/perl -w 
#
# DateTime_test.pl -- test the DateTime.pm class
#

use strict;
use lib "$ENV{'HOME'}/perllib";

use DateTime;

print "\nDateTime:  start testing . . .\n";
print "test1:  Should get J2000 epoch for new uninitialized DateTime object.\n";
my $dt = new DateTime;
print $dt->toCalStr, "\n";

print "\ntest2:  Init date/time object to 513246.1234 sec. and print.\n";
$dt = new DateTime(513246.1234);
print $dt->toCalStr, "\n";
my $cal = `sec2cal 513246.1234`;
print "sec2cal 513246.1234 = ", $cal;

print "\ntest3:  Set date/time to 2001 10 06 01 02 03 .456789 and print in various formats.\n";
$dt = (new DateTime)->fromCal( 2001, 10, 06, 01, 02, 03, 0.456789 );
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";
my $sec = `cal2sec 2001 10 06 01 02 03 .456789`;
print "cal2sec 2001 10 06 01 02 03 .456789 = ", $sec;

print "\ntest4:  Repeat test 3 but init with strings rather than integers.\n";
$dt->fromCal( "2001", "10", "06", "01", "02", "03", "0.456789" );
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest5:  Set date/time to 1999 02 28 09 08 07 .654321 and print in various formats.\n";
$dt = (new DateTime)->fromCal( 1999, 2, 28, 9, 8, 7, .654321 );
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";
$sec = `cal2sec 1999 2 28 9 8 7 .654321`;
print "cal2sec 1999 2 28 9 8 7 .654321 = ", $sec;

print "\ntest6:  Set date/time to string '2001/10/06,01:02:03.456789' and print in various formats.\n";
$dt->fromCalStr("2001/10/06,01:02:03.456789");
print "ChStr = ", $dt->toCh, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest7:  Set date/time to string '06-oct-2001 01:02:03.456789' and print in various formats.\n";
$dt->fromCh("06-oct-2001 01:02:03.456789");
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest8:  Set date/time to string '1999/02/28,09:08:07' (no frac part) and print in various formats.\n";
$dt->fromCalStr("99/02/28,09:08:07");
print "ChStr = ", $dt->toCh, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest9:  Set date/time to string '28-FEB-1999 09:08' (no sec or frac part) and print in various formats.\n";
$dt->fromCh("28-FEB-1999 09:08");
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest10:  Add delta of -901. seconds to date/time and print in various formats.\n";
$dt->addDeltaSec(-901.);
print "CalStr = ", $dt->toCalStr, "\n";
print "ChStr  = ", $dt->toCh, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest11:  Add delta time of -2/01:30:02.345 seconds to date/time and print in various formats.\n";
$dt->addDeltaTime("-2/01:30:02.345");
print "CalStr = ", $dt->toCalStr, "\n";
print "ChStr  = ", $dt->toCh, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\ntest12:  Test delta2sec routine.\n";
my $delta = "-/2:30:02";
print "Delta Time = ", $delta, "\n";
$sec = DateTime::delta2sec($delta);
print "Sec = ", $sec, "\n";

print "\ntest13:  Test sec2delta routine.\n";
$sec = 98734.5678;
print "Sec = ", $sec, "\n";
my $DeltaTime = DateTime::sec2delta($sec);
print "DeltaTime = ", $DeltaTime, "\n";
print "delta2sec $DeltaTime = ", DateTime::delta2sec($DeltaTime), "\n";

print "\ntest14:  Set date/time to string '28-FEA-1999 09:08:07' (misspelled month str) and print in various formats.\n";
$dt->fromCh("28-FEA-1999 09:08:07");
print "CalStr = ", $dt->toCalStr, "\n";
print "Sec = ", $dt->toSec, "\n";

print "\nDateTime:  done testing.\n\n";
