#!/usr/local/bin/perl -w
#
# SvnPrn.pm -- Lookup table for SVN to PRN conversion, or vica-versa.
#

package SvnPrn;
use strict;
use Carp;

# Implemented by reading lookup table into two hashes.
# Both sets of numbers are kept as two-character strings.
#
# Use:
#   svn_to_prn -- to convert
#   prn_to_svn -- to convert back


sub new { # construct lookup tables by reading from the file argument
    my($class, $file) = @_;
    my %h = ();

    if ( defined $file && scalar $file ) {
	my $self = bless( \%h, ref($class) || $class );
	$self->init($file);
	return $self;
    } else {
        croak "SvnPrn->new(): One arg should be file name, not: $file";
    }
}

sub init { # construct lookup tables by reading from the file argument
    my($self, $file) = @_;
    
    open(IN, $file) or croak "SvnPrn->init(): Cannot open file: $file";
    $$self{'svn2prn'} = ();
    $$self{'prn2svn'} = ();
    my ($svn, $prn);
    while ( <IN> ) {
	next if /^#/;
	($svn, $prn) = split;
        if ( length($svn) != 2 ) { $svn = sprintf "%2.2d", $svn; }	
        if ( length($prn) != 2 ) { $prn = sprintf "%2.2d", $prn; }	
	$$self{'svn2prn'}{$svn} = $prn;
	$$self{'prn2svn'}{$prn} = $svn;
    }
}

sub svn2prn { # convert from SVN to PRN (as two-character strings)
    my($self, $key) = @_;
    my $val;

    if ( length($key) != 2 ) { $key = sprintf "%2.2d", $key; }
    if ( defined ($val = $$self{'svn2prn'}{$key}) ) {
	return $val;
    } else {
	carp "SvnPrn->svn2prn(): No PRN value for SVN = $key";
    }
}

sub prn2svn { # convert from PRN to SVN (as two-character strings)
    my($self, $key) = @_;
    my $val;

    if ( length($key) != 2 ) { $key = sprintf "%2.2d", $key; }
    if ( defined ($val = $$self{'prn2svn'}{$key}) ) {
	return $val;
    } else {
	carp "SvnPrn->prn2svn(): No SVN value for PRN = $key";
    }
}

1;
