#!/usr/local/bin/perl -w
#######################
# Lukas Mandrake
#######################
# An object that handles bi-directional piping
#######################


package Two_Way_Pipe;

#use lib "/usr/local/lib/perl5/5.8.0/IPC/";
use IPC::Open2;
use strict;

sub new
 {
  my $read_pipe;
  my $write_pipe;
  my $process;
  my $pid;

  my $self = bless ([$read_pipe, $write_pipe, $process, $pid],shift);
  Two_Way_Pipe::connect ($self,@_);

  return $self;
 }

sub connect
 {
  my ($self, $process) = @_;

  if (! $process) { return; }

  my ($read_pipe, $write_pipe, $old_process) = @$self;
  if ($old_process) { Two_Way_Pipe::disconnect($self); }

  my $pid = open2(\*read_pipe, \*write_pipe,$process);

  @$self = (\*read_pipe, \*write_pipe, $process, $pid);
 }

sub disconnect
 {
  my $self = shift;

  my ($read_pipe, $write_pipe, $process, $pid) = @$self;

  if ($read_pipe) 
   {
    close read_pipe;
    close write_pipe;
    waitpid($pid,0);
    @$self = (undef, undef, undef, undef);
   }
 }

sub write
 {
  my ($self, $string) = @_;
  my ($reader, $writer, $process, $pid) = @$self;

  print $writer $string,"\n";
 }

sub read_line
 {
  my $self = shift;
  my ($reader, $writer, $process, $pid) = @$self;

  my $readln = <$reader>;

  return $readln;
 }


1;
