#!/usr/local/bin/perl -w
#
# Util.pm -- Catch all module for random utility functions.
#

package Util;
use Exporter ();
@ISA = qw(Exporter);
@EXPORT    = qw(eex echo echo2);
@EXPORT_OK = qw(echo2_exec new_file_version assoc_str_replace parse_file_name);

use strict;
use Carp;


sub echo {  # Echo a string to STDOUT.
    my $str = shift @_;
    print STDOUT $str, "\n";
}


sub echo2 {  # Echo a string to STDERR.
    my $str = shift @_;
    print STDERR $str, "\n";
}


sub eex {  # Echo a command to STDERR and then exec it.
    my $cmd = shift @_;

    print STDERR $cmd, "\n";
    `$cmd`;
    carp $@ if $@;
    return $@;
}


sub echo2_exec {  # Echo a command to STDERR and then exec it.
    eex @_;
}


sub new_file_version {  # Generate a new unique file name by adding an extension.
                        # Mode = 'a' for appending a..z, 'n' for appending '.1', '.2', etc.
  my($file, $mode) = @_;

  my (@exts, $sep);
  if ( defined $mode && $mode =~ /a/i ) {
      @exts = ('a' .. 'z');
      $sep = "";
  } else {
      @exts = (1 .. 99);
      $sep = '.';
  }

  my $new_file = $file;
  my $ch;
  while ( -e $new_file ) {
      ($ch = shift @exts) || croak "Util::new_file_version: Ran out of file verions.";
      $new_file = $file . $sep . $ch;
  }
  return $sep . $ch;
}


sub assoc_str_replace {
    my $rhash = shift @_;
    my ($dir, $tmp_file);

    while ( my $file = shift @_ ) {
        ($dir) = parse_file_name($file, "dir");
	$tmp_file = "${dir}tmp$$";
	open(OUT, "> $tmp_file") || croak "Util::assoc_str_replace: Cannot write into directory, $dir";
	open(IN, $file) || croak "Util::assoc_str_replace: Cannot open file, $file";
	while ( <IN> ) {
            foreach my $key ( keys %$rhash ) {
                $_ =~ s/$key/$$rhash{$key}/g;
	    }
	    print OUT $_;
	}
	close(IN);
	close(OUT);
	rename $file, "$file.bak";
	rename $tmp_file, $file;
    }
    unlink $tmp_file;
}


sub parse_file_name {
    my ($fn, $parts) = @_;
    my ($dir, $root, $ext, @parts);

    if ( index($fn, "/") >= 0  ) {
        if ( $fn !~ m#^(.*/)([^/]+)(\.?[^/]*)$# ) {
            croak "Util::parse_file_name: Improper Unix file name, $fn";
        } else {
            $dir  = defined $1 ? $1 : "";
            $root = defined $2 ? $2 : "";
            $ext  = defined $3 ? $3 : "";
	}
    } else {
	$dir = "";
        if ( $fn !~ m#^([^/]+)(\.?[^/]*)$# ) {
            croak "Util::parse_file_name: Improper Unix file name, $fn";
        } else {
            $root = defined $1 ? $1 : "";
            $ext  = defined $2 ? $2 : "";
	}
    }
    push @parts, $dir  if $parts =~ /dir/i;
    push @parts, $root if $parts =~ /root/i;
    push @parts, $ext  if $parts =~ /ext/i;
    return @parts;
}


sub generate_file_time_stamp {
    # Given date (YYMMDD) and time in minutes since start-of-day,
    # generate a string time stamp suitable for file name.
    my ($short_date, $min_ssod) = @_;

    my $hr = int($min_ssod / 60.);
    my $hr_str = sprintf("%2.2d", $hr);
    my $min = $min_ssod - $hr * 60.;
    my $min_str = sprintf("%2.2d", $min);
    my $time_stamp = $short_date . "_" . $hr_str . $min_str;
    return $time_stamp;
}         


1;
