#!/usr/local/bin/perl -w 
# file get_prompt_geomag_indices.pl

if ( @ARGV < 1 ) { print STDERR <<EOS;  exit }
usage: get_prompt_geomag_indices.pl  <date, YYYY/MM/DD>

EOS

use strict;
use LWP;

use constant SEC_SWx_SITE => 'http://sec.noaa.gov/ftpdir/indices/';
use constant SEC_DGD   => 'DGD.txt';       # daily geomagnetic data
use constant SEC_DSD   => 'DSD.txt';       # daily solar data

(my $date = shift) =~ s#/# #g;         # date now contains YYYY MM DD
print "Date: $date\n";

# Find Ap & Kp indices in the DGD file for the specified date.
my $agent = LWP::UserAgent->new;
my $url = SEC_SWx_SITE . SEC_DGD;
my $request = HTTP::Request->new( GET => $url );
my $response = $agent->request($request);
#print $response->code, "\n";
print $response->content, "\n";
$response->is_success or die "$url: ", $response->message, "\n";

my @lines = split( "\n", $response->content);
my $found_Ap = 0;
my $found_hdr = 0;
my $year = substr( $date, 0, 4);
my $Ap = "-1";
my @indices;
my @Kp = qw(-1 -1 -1 -1 -1 -1 -1 -1);
for ( my $i = 0; $i <= $#lines; $i++ ) {
    $_ = $lines[$i];
    if ( !$found_hdr && /^$year/ ) {
	for ( my $j = -3; $j < 0; $j++ ) { print $lines[$i + $j], "\n" }
	$found_hdr++;
    }
    if ( /^$date/ ) {
	print $lines[$i], "\n";
	@indices = split;
	if ( $indices[3] ne "-1" ) {
            $Ap = $indices[21] || "-1";
	    @Kp = (@indices)[22,23,24,25,26,27,28,29];
	    $found_Ap++;
	}
    }
}

# Find F10.7 flux in the DSD file for the specified date.
$url = SEC_SWx_SITE . SEC_DSD;
$request = HTTP::Request->new( GET => $url );
$response = $agent->request($request);
#print $response->code, "\n";
print $response->content, "\n";
$response->is_success or die "$url: ", $response->message, "\n";

@lines = split( "\n", $response->content);
my $found_F10_7 = 0;
$found_hdr = 0;
my $F10_7 = "-1";
my $SSN = "-1";
print "\n";
for ( my $i = 0; $i <= $#lines; $i++ ) {
    $_ = $lines[$i];
    if ( !$found_hdr && /^$year/ ) {
	for ( my $j = -4; $j < 0; $j++ ) { print $lines[$i + $j], "\n" }
	$found_hdr++;
    }
    if ( /^$date/ ) {
	print $lines[$i], "\n";
	@indices = split;
	$F10_7 = $indices[3] || "-1";
	$SSN = $indices[4] || "-1";
	$found_F10_7++;
    }
}

print "\nAp: $Ap\n";
print join " ", "Kp (3-hour):", @Kp, "\n";
print "F10.7: $F10_7\n";
print "SSN: $SSN\n";

$found_Ap or die "get_prompt_geomag_indices.pl: Geomag indices not found for this date.";
$found_F10_7 or die "get_prompt_geomag_indices.pl: Solar indices not found for this date.";

1;
