#!/usr/local/bin/perl
# file get_radiosonde_data.pl

if ( @ARGV < 3 ) { print STDERR <<EOS;  exit }
usage: get_radiosonde_data.pl <beg_date_hour, YYYY/MM/DD:HH>  <end_date_hour> <wmo's . . .>

EOS

use strict;
use LWP;
use HTTP::Request::Common;

use constant RAOB_SITE => 'http://raob.fsl.noaa.gov/intl/';
use constant RAOB_CGI  => 'GetRaobs.cgi';

(my $bdate = shift) =~ tr#/:##d;          # convert YYYY/MM/DD:HH to YYYYMMDDHH
(my $edate = shift) =~ tr#/:##d;
my $wmo_list = join " ", @ARGV;

my $agent = LWP::UserAgent->new;

my $request = POST( RAOB_SITE . RAOB_CGI,
                   Content => [ shour   => 'All Times',
				ltype   => 'All Levels',
				wunits  => 'Knots',
				bdate   => $bdate,
				edate   => $edate,
				access  => 'WMO Station Identifier',
				view    => 'NO',
				StationIDs => $wmo_list,
				osort   => 'Station Series Sort',
				oformat => 'FSL format (ASCII text)'
                              ] );

my $response = $agent->request($request);
#print $response->code, "\n";
#print $response->content, "\n";

if ( $response->code == 302 ) {              # then have been redirected to data file
    $response->content =~ m/document has moved <A HREF="(.*)">here/is
	or die "get_radiosonde_data.pl: Cannot find name of data file.  Wrapper broken.";
    my $url = RAOB_SITE . $1;

    $request = HTTP::Request->new( GET => $url );
    $response = $agent->request($request);
    $response->is_success or die "$url: ", $response->message, "\n";
    print $response->content;

} elsif ( $response->code == 200 ) {
    if ( $response->content =~ m/No data is currently available/is ) {
	die "No data for this site and period";
    } else {
	die "get_radiosonde_data.pl: Errror, unexpected HTML result.  Wrapper broken.";
    }

} else {
#    $response->is_success or die "RAOB_GET: ", $response->message, "\n";
    die "RAOB_GET: ", $response->message, "\n";
}

