#!/usr/local/bin/perl -w
#
# lukeutils.pm contains usefil function calls originally used for GIM validation purposes
#

use strict;
use Carp;

package lukeutils;

################################################################################
#Figures out which of a set of files exists and returns that path
################################################################################
sub which_one_works
{
 foreach (@_) { if (-s $_) {return $_; }}
}

################################################################################
#Waits until a disfavorable return value from an eval does not occur, retrying
#the run periodically. Useful for license busy failures.
################################################################################
sub waitforit
{
 my ($program, $bad_result, $notice_message, $waittime)=@_;
 my $result;

 while (grep /$bad_result/,$result=`$program`) {
  print "$notice_message, waiting $waittime sec...\n"; sleep $waittime; 
 }
 return $result;
}

################################################################################
#Removes a single character (or more than one) from the end of a string
################################################################################
sub trim
 {
  my ($string,$num)=@_;
  my $truenum = (defined $num) ? $num : 1;
  
  return substr($string,0,length($string)-$truenum);
 }


################################################################################
#Finds the index at which a substring occurs in a string array
################################################################################
sub array_index
 {
  my ($object,$rArray) = @_;
  for (my $t=0;$t<@$rArray;$t++) { if ($object eq substr($$rArray[$t],0,length($object))) { return $t; } }
  return -1;
 }

################################################################################
#Sorts a given (arbitrarily long) string array in order given by a second array#
################################################################################
sub sort_like_this
 {
  my ($rMaster,$rOrder) = @_;
  
  @$rMaster = map { $_->[0] }
              sort { $a->[1] <=> $b->[1] }
              map {[$_,array_index((split ' ',$_)[1],$rOrder)]} @$rMaster;
 }

################################################################################
#Waits for specified file to be defined with a non-zero length.
#Arguments: "full path",(seconds to wait)
################################################################################
sub wait_for_file
 { 
  my ($filename, $seconds_to_sleep)=@_;
  while (! (-s $filename) ) { print "Waiting $seconds_to_sleep s for $filename...\n"; sleep $seconds_to_sleep; } 
 }

################################################################################
#Wraps a 24 hour time value back into the range 0-24 while maintaining decimals
################################################################################
sub clock_wrap
 { 
  my $bigclock=shift;
  return $bigclock %24+($bigclock-int($bigclock)); 
 }

################################################################################
#Tests for a valid number in the passed string. Permits fortran notation.
################################################################################
sub valid_number
 {
  my $value=shift;
  if (!$value) {return 0;}
  return $value=~/^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/;
 }

################################################################################
#Handy subroutines that don't fit in any particular objects but may be used
#by all of them.
################################################################################
##### Call is: file_to_array (filename, line count to preallocate <optional>)
sub file_to_array
 {
  use Carp;

  my $warning;
  my @result;
  $result[$_[1]-1]=0 unless scalar(@_)<2;
  if ((-s $_[0]) == 0) { croak "$_[0] has zero length\n"; }
  open (FILER,$_[0]) || croak "Can't open $_[0]: $!";
  @result=<FILER>;
  chomp(@result);  
  close(FILER);
  return \@result;
 }
################################################################################
#Rename all files in a directory according to a name substitution
################################################################################
sub rename_all_files #directory, start pattern, end pattern
{
 use Carp;

 my ($directory,$search,$replace)=@_;
 opendir(FILES,$directory) || croak "Cannot open $_[0]: $!";
 my $temp;
 my $search_pattern;
 while ($temp=readdir(FILES))
 {
  chomp($temp);
  if ($temp=~/$search/)
  { my $indexer;
    $indexer=$temp;
    $indexer=~s/$search/$replace/;
    rename($directory."/".$temp,$directory."/".$indexer);
  }
 }
 closedir(FILES);
}

################################################################################
#Eliminate any redundant entries in an array (sorts array first)
################################################################################
sub make_unique_array #array reference
{
 my $array=shift;
 my $size=scalar(@$array);
 @$array=sort @$array;

 for (my $t=0;$t<$size-1;$t++)
  {
   if ($$array[$t] eq $$array[$t+1])
    {
     my @temp;
     @$array=(@$array[0..$t],@$array[$t+2..scalar(@$array)]);
     $size--;
     $t--;
    }
  }
 my @temp=@$array[0..$size-1];
 undef @$array;
 @$array=@temp;
}

#@unique = grep { ++$count{$_} < 2 } 
#               qw(a b a c d d e f g f h h);
#print "@unique\n";
#
#a b c d e f g h


1;

