#!/usr/local/bin/perl -w 
#
# merge_rinex_obs.pl -- Merge multiple RINEX OBS files into a single file sorted by time.
#                       Can exclude records before a specified begin time and after a specified end time.
#                       Does a proper merge even if files or records are out of time order.
#

use strict;
use lib "~bdw/perllib";
#use lib "$ENV{'SOURCE'}/perllib";
use Rinex;
use File::Basename;

die "Usage: merge_rinex_obs.pl -norm -beg 2001/08/23,18:00:00 -end 2001/08/24,13:30:30 -o <merged filename> <RINEX obs files> . . ."
    unless @ARGV >= 1;

my $begin_time = "";
my $end_time = "";
my $out_file = "";
my $normalize_flag = 0;
my ($file1_in, @files);

my $more_switches = 1;
while ( $more_switches ) {
    $_ = shift @ARGV;
    if ( m/^-/ ) {
	if ( m/^-beg/ ) {
	    $begin_time = shift @ARGV;
	} elsif ( m/^-end/ ) {
	    $end_time = shift @ARGV;
	} elsif ( m/^-n/ ) {
	    $normalize_flag = 1;
	} elsif ( m/^-o/ ) {
	    $out_file = shift @ARGV;
	} else {
	    die "merge_rinex_obs.pl: Illegal switch => $_";
	}
    } else {
        unshift @ARGV, $_;
        $more_switches = 0;
    }
}

if ( $normalize_flag ) {
    $file1_in = $ARGV[0];                # save first input file name
    Rinex::normalize_files( \@ARGV ) if @ARGV > 1;
    @files = @ARGV;                      # save possibly new file names to clean up
}

if ( $out_file ne "" ) {
    open(OUT, "> $out_file") || die "merge_rinex_obs.pl: Cannot open output file $out_file";
    Rinex::merge_rinex_obs($begin_time, $end_time, \*OUT, \@ARGV);
} else {
    Rinex::merge_rinex_obs($begin_time, $end_time, \*STDOUT, \@ARGV);
}

if ( $normalize_flag ) {
    if ( $file1_in ne $files[0] ) {    # normalize of RINEX files was NOT in place, so clean up files and tmp dir
        unlink @files;
        my($fn, $path, $ext) = fileparse($files[0]);
        unlink $path;
    }
}
