#!/usr/local/bin/perl -w 
#
# ping_hosts.pl -- Ping a list of hosts to see what computers are up.
#

use strict;

my $PING_CMD = "/usr/sbin/ping \$host -n";    ## for HP's
my $NSLOOKUP_CMD = "/bin/nslookup";

die "Usage: ping_hosts.pl "
    unless @ARGV >= 1;

my $more_switches = 1;
while ( $more_switches ) {
    $_ = shift @ARGV;
    if ( m/^-/ ) {
	if ( m/^-beg/ ) {
#	    $begin_time = shift @ARGV;
	} elsif ( m/^-end/ ) {
#	    $end_time = shift @ARGV;
	} else {
	    die "ping_hosts.pl: Illegal switch => $_";
	}
    } else {
#        $in_file = $_;
        $more_switches = 0;
    }
}

#my @hosts = map "137.79.31.$_", "001" .. "254";
my @hosts = map "137.79.31.$_", "027" .. "028";
my %pid_to_host;
my %host_result;

for my $host (@hosts) {
    wait_for_a_kid() if keys %pid_to_host > 10;
    if ( my $pid = fork) {
	## parent
	$pid_to_host{$pid} = $host;
#	warn "$pid is processing $host\n";
    } else {
	## child
	exit !ping_a_host($host);
    }
}

## final reap
1 while wait_for_a_kid();


for my $host (sort keys %host_result) {
    (my $host_name = `$NSLOOKUP_CMD $host`) =~ s/Name: *([a-z0-9.])/$1/i;
    $host_name = $host if !$host_name;
    print "$host is ", ($host_result{$host} ? "good" : "bad"), "\n";
}


# Subroutines follow.

sub ping_a_host {
    my $host = shift;
    `$PING_CMD` =~ /0 packets received/ ? 0 : 1;
}

sub wait_for_a_kid {
    my $pid = wait;
    return 0 if $pid < 0;
    
    my $host = delete $pid_to_host{$pid}
            or warn("Why did I see $pid ($?)\n"), next;
#    warn "reaping $pid for $host\n";
    $host_result{$host} = $? ? 0 : 1;
    1;
}

