#!/usr/local/bin/perl
if (! @ARGV) {
  print <<EOF;

  usage:  smallgim <gimfile> ...

  Reduces the size of the specified GIM files by removing redundant
  characters and excess precision.

EOF
  exit(2);
}
while (@ARGV) {
  $file = shift;
  ($fmt,$type,$size,$order) = &whatkind($file);
  if ($fmt ne "old") {
    if ($fmt eq "unk") {
      warn "unknown GIM file format, skipping $file\n\n";
    } elsif ($fmt eq "new") {
      warn "$file is already converted\n\n";
    }
    next;
  }
  open (IN,"< $file");
  open (OUT,"> tempGIM") || die "cannot open tempGIM\n";
  $ct = 0;
  $timect = 0;
  while (<IN>) {
    chomp;
    @itm = split();
    $ct++;
    if ($itm[0] =~ "^!" || $itm[0] eq "STOP") {
      print OUT "$_\n";
      next;
    }
    if ($itm[0] eq "TIME") {
      if ($itm[2] =~ /20671231/) {
	  substr($itm[1],11,2) += $span;
          (s/$itm[2]/$itm[1]/);
      } else {
	$span = &time($itm[2])- &time($itm[1]);
        if ($span < 0) {$span += 86400;}
	$span = int($span/60+.5);
      }
      $save = $_;
      $block = 1;
      $line = 0;
      $timect++;
      next;
    }
    $type = substr($itm[0],0,4);
    if (length($save)) {
      print OUT "$save $type $size $order\n";
      $save = "";
    }
    if (length($itm[0]) > 4) {
      $i = substr($itm[0],4);
    } else {
      $i = "1";
    }
    $line++;
    if ($line != $itm[1] || $i != $block) {
      print "loss of line synch at line $ct, time block $timect\n";
      print "expected coeff $line,$block, found $itm[1],$i\n";
      exit(2);
    } elsif ($line == $size) {
      printf OUT "%11.5f %7.2f %d\n", $itm[2],$itm[3],$i;
      $block++;
      $line=0;
    } else {
      printf OUT "%11.5f %7.2f\n", $itm[2],$itm[3];
    }
  }
  close (IN);
  close (OUT);
  rename tempGIM,$file || die "cannot rename tempGIM to $file\n";
}
sub whatkind {
  my ($fmt,$type,$size,$order);
# open each file, find the first TIME line, and split it.

  open (IN,"< $_[0]") || die "cannot open $_[0]\n";
  while (! /^TIME/ && ! eof(IN)) {
    $_=<IN>;
    chomp;
  }
  @itm = split();

# if more than "TIME yyyymmdd-hhmmss yyyymmdd-hhmmss", file is
# already converted, give up
  if (@itm > 3 ) {
    if ($itm[3] =~ /^(HARM|TRIN|TAVG|SPC2|GRID)/) {
      $type = $&;
      $fmt = "new";
      $size = $itm[4];
      $order = $itm[5];
    } else {
      $fmt = "unk";
      $typ = "unk";
      $size = 0;
      $order = 0;
    }
    return ($fmt,$type,$size,$order);
  }
# read the next line and split it
  $_ = <IN>;
  chomp;
  @itm = split();
# check for a known coefficient type, save if found
# or skip file if not
  if ($itm[0] =~ /^(HARM|TRIN|TAVG|SPC2|GRID)/) {
    $type = $&;
  } else {
    $fmt = "unk";
    $typ = "unk";
    $size = 0;
    $order = 0;
    return ($fmt,$type,$size,$order);
  }
  $line =0;
# loop until next TIME line, saving highest order
# and size of coefficient array
  while ($itm[0] ne "TIME") {
    $line++;
    $itm[0] =~ /$type/;
    $order = $';
    if ($itm[1] > $size) {$size = $itm[1];}
    $_=<IN>;
    chomp;
    @itm = split();
  }
  close(IN);
  return ("old",$type,$size,$order);
} # end subroutine whatkind
sub time {
  return (substr($_[0],9,2)*60+substr($_[0],11,2))*60+substr($_[0],13,2);
}



