#!/usr/local/bin/perl
# file soap_plot_stlocs.pl

if ( @ARGV != 1 ) { print STDERR <<EOS;  exit }
usage: soap_plot_stlocs.pl <STLOCS file>

EOS

use strict;
use HTTP::Daemon;
use SOAP::Lite;
#use HTTP::Request::Common;

use constant RAOB_GET      => 'http://raob.fsl.noaa.gov/intl/GetRaobs.cgi';

(my $bdate = shift) =~ tr#/:##d;          # convert YYYY/MM/DD:HH to YYYYMMDDHH
(my $edate = shift) =~ tr#/:##d;

my $agent = LWP::UserAgent->new;

my $request = POST( RAOB_GET,
                   Content => [ shour   => 'All Times',
				ltype   => 'All Levels',
				wunits  => 'Knots',
				bdate   => $bdate,
				edate   => $edate,
				access  => 'WMO Station Identifier',
				view    => 'YES',
				osort   => 'Station Series Sort',
				oformat => 'FSL format (ASCII text)'
				] );

my $response = $agent->request($request);
$response->is_success or die "RAOB_GET: ", $response->message, "\n";
#print $response->content;
my @lines = split( "\n", $response->content);

# Find 'SELECT NAME=station list' line in the returned HTML.
my $inside = 0;
my $j;
for ( my $i = 0; $i < $#lines; $i++ ) {
    if ( $lines[$i] =~ /<select name="station_list"/i ) {
	$inside++;
	$j = ++$i;
	last;
    }
}
$inside or die "get_radiosonde_sites.pl: No station list on RAOB web page.  Wrapper broken.";

# Find all of the '<OPTION>' lines that contain site names, wmos, lats, and lons,
# and are also *inside* the select.
for ( my $i = $j; $i < $#lines; $i++ ) {
    $_ = $lines[$i];
    if ( /^[\s\t]*<OPTION>/i ) {
	(my $stn, my $wmo, my $lat, my $lon, my $name) = (split)[1, 3, 4, 5, 7];
	$stn .= " " x (4 - length($stn));
        # print unique WMO number, stn_name, lat, lon, full_name
#	print $_, "\n";
	print join " ", $wmo, $stn, $lat, $lon, $name, "\n";
    }
    last if /<\/SELECT>/i;
}
