#!/usr/local/bin/perl -w
#

use strict;

################################################################################
package Statistics;
################################################################################
#Keeps running accumulation of various statistics combined with labels for 
#each data point providing convenient min/max tracking.
################################################################################

                 #number, mean,sum(squares), max, min,  maxlabel, minlabel
sub new { bless ([     0, 0.0,         0.0, 0.0, 0.0,"mxlbl","mnlbl"] ,shift); }

sub add {
  my $self=shift;
  if (scalar(@_)<1) { return 0; }
  my ($label, $val) = (scalar(@_)>1? $_[1]:"" , $_[0]);

    $self->[0]++;
    my $dval = $val - $self->[1];  # difference from current mean
    $self->[1] += $dval/$self->[0]; # update mean
    my $ddval = $val - $self->[1];
    $self->[2] += $dval*$ddval;

    if ( $self->[0] == 1 ) {
 	 $self->[3] = $val;
 	 $self->[4] = $val;
         $self->[5] = $label;
         $self->[6] = $label; 
    } else {
        if ($val>$self->[3]) {$self->[3] = $val; $self->[5]=$label;}
        if ($val<$self->[4]) {$self->[4] = $val; $self->[6]=$label;}
    }


}

sub calc {
  my $self = shift;
  my($n, $mean, $sumsq, $max, $min, $maxlabel, $minlabel) = @{ $self };
  my($sigma, $rms) = (0, 0, 0);

    if ($n == 0 ) { return (0,0,0,0,0,0,"",""); }
    if ( $n < 2 ) { $sigma = 0; }
    else {
     my $sd2 = $sumsq/($n-1);
     $sigma = ($sd2 <= 0) ? 0 : sqrt($sd2);
     }
    $rms = sqrt( $mean*$mean + $sumsq/$n);

  ($n, $mean, $sigma, $min, $max, $rms, $minlabel, $maxlabel);
}

sub reset { my $self = shift; @$self = (0,0.0,0.0, 0.0, 0.0,"maxlbl","mnlbl"); }

sub add_many   #must pass (array_values, array_labels)
 {
  use Carp;
  my ($self, $values_ref, $labels_ref) = @_;
  
  my $t;
  $t=0;
  my $label;
  foreach (@$values_ref)
   {
    $label = defined $labels_ref? $$labels_ref[$t]:"";
    add($self,$_, $label);
    $t++;
   }
  return $t;
 }

1;
