#!/usr/local/bin/perl -w -d
#
# write_rinex_obs.pl -- Write out RINEX OBS file as columns of numbers sorted by sat and then time.
#

use strict;
use lib "~bdw/perllib";
use Rinex2;

die "Usage: write_rinex_obs.pl -beg 2001/08/23,18:00:00 -end 2001/08/24,13:30:30 -o <output file> <RINEX obs file>"
    unless @ARGV >= 1;

my $begin_time = "";
my $end_time = "";
my $out_file = "";
my $in_file;
my $augment = 0;
my $connect = 0;

my $more_switches = 1;
while ( $more_switches ) {
    $_ = shift @ARGV;
    if ( m/^-/ ) {
	if ( m/^-beg/ ) {
	    $begin_time = shift @ARGV;
	} elsif ( m/^-end/ ) {
	    $end_time = shift @ARGV;
	} elsif ( m/^-o/ ) {
	    $out_file = shift @ARGV;
	} elsif ( m/^-a/ ) {
	    $augment = 1;
	} elsif ( m/^-c/ ) {
	    $connect = 1;
	} else {
	    die "merge_rinex_obs.pl: Illegal switch => $_";
	}
    } else {
        $in_file = $_;
        $more_switches = 0;
    }
}

open(IN, $in_file) || die "write_rinex_obs.pl: Cannot open input file $in_file";
print STDERR "Rinex2::write_rinex_obs: Reading ", $in_file, " . . .\n";

my %hdr = ();
my @blk = ();
my $n_time_epochs = Rinex2::read_obs(\*IN, \%hdr, \@blk);

my $r_blk = Rinex2::time_filter_obs($begin_time, $end_time, \@blk);
@blk = ();

if ( $out_file ne "" ) {
    open(OUT, "> $out_file") || die "merge_rinex_obs.pl: Cannot open output file $out_file";
    Rinex2::write_obs(\*OUT, \%hdr, $r_blk, $augment, $connect);
} else {
    Rinex2::write_obs(\*STDOUT, \%hdr, $r_blk, $augment, $connect);
}
