#!/usr/local/bin/python 
# Lukas Mandrake

import os, sys
from ionoutil import *
from optparse import OptionParser
from sys import argv
#import pdb
import ionotime as I
#import coordinates as C

def main():

 USAGE = """
%prog file1.profile file2.profile ... <switches>

Explores a GAIM _pql solution and extracts GAIM results
for all .profile files specified, generating .profile2
files. Also generates a latlon.critical file containing critical
parameters versus time.

"""

 parser = OptionParser(usage=USAGE)

 parser.add_option('-i','--ionosonde',type="string",default="99,99",
   help="Will read lat/lon pair seperated by comma and generate that profile independently")
# parser.add_option('-a','--arcseparation',type="float",default=0,
#   help="If consecutive data points in an arc are beyond ARCSEPARATION (sec) separated in time, consider it a new arc. Default 20 minutes.")

 (options, args) = parser.parse_args()

 if not args and not options: die("Must specify lat and lon or profile file")

 #figure out which times to request based on
 ex("ls -1 gaim_state_pql*.mat > files_to_process.list")
 times=[I.IonoTime(x[-19:-4]) for x in getdir("gaim_state_pql*.mat")]

 #establish connection to gaim_extract.e (fortran)
 e=os.environ
 (child_stdin,child_stdout)=os.popen2(e['GAIM']+"/build/"+e['ARCH']+"/"+e['MODE']+"/prg/postpro/ncov/gaim_extract.e")

 #loop over all times of state files (if lat/lon specified)
 if not options.ionosonde == "99,99":
     (lat,lon)=[float(x) for x in options.ionosonde.split(",")]
     if abs(lat)>89.9 or abs(lon)>360: die("Illegal latitude or longitude: "+options.ionosonde)
     print "Activating lat/lon profile extraction... will generate profiles for all times found"
     for d in sorted(times):
         print "Processing profile for time: "+d
         child_stdin.write("%f %f %f\n"%(d.to("J2000"),lat,lon))
         child_stdin.flush()
         f=open("gaim_profile_lat%03d_lon%03d_"%(lat,lon)+d+".profile","w")
         altn=child_stdout.readline().split(",")

         #find critical parameters
         (hmf2,nmf2)=find_pair_max([[float(z) for z in y.split()] for y in altn])
         f.write(make_header("profile","gaim.profile","HMF2:%05.1f,NMF2:%05.5e"%(hmf2,nmf2))+"\n")

         #write results
         for L in altn:
             (alt,n)=[float(x) for x in L.split()]
             f.write("%11.1f %05.0f %4.1f %4.1f %5.5e 0 GAIM %6.1f %8.4f %8.4f %s%s\n"%(d.to("J2000"),d.to("secondsinday"),
                                                                                I.hms_to_hours(d.to("localhms",lon)),
                                                                                d.to("hoursinday"),n,alt,
                                                                                lat,lon,
                                                                                "GAIM%03d%03d"%(lat,lon),d.to("YYMMDD_HHMMSS")))
         f.close()
 else:
 #loop over all .profile files specified and append GAIM results
     date=None
     for f in sorted(args):
         print "Augmenting "+f
         oldppt=(0,0,0)
         max_ne=max_height=0
         output=""
         oldheader=""
         for L in open(f):
             if L[0]=="#":
                 if L[:4]=="#PRO":
                     oldheader=L.split("_")[5].replace("foruse:","").strip()
                 continue
             (time,sod,lt,hd,n,Te,inst,alt,lat,lon,ID)=L.strip().split()
             time,n,alt=[float(x) for x in (time,n,alt)]
             if time > max(times):
                 warn("Requested time of %s which is beyond max state file of %s, skipping"%(I.IonoTime(time),max(times)))
                 continue
             date=I.IonoTime(time)
             print "Processing profile for time:"+date
             if not oldppt == (lat,lon,time): #only load in a new profile if it's different in lat,lon, or time
#                 print "%f %s %s\n"%(time,lat,lon)
                 child_stdin.write("%f %s %s\n"%(time,lat,lon))
                 child_stdin.flush() #force write to fortran
                 gaim_profile=[x.strip().split() for x in child_stdout.readline().strip().split(",")] #process gaim_extract output
                 gaim_profile=[(float(x),float(y)) for x,y in gaim_profile] #convert to float pairs
                 oldppt = (lat,lon,time)
             gaim_ne=interpolate_pair_array(gaim_profile,alt,0.0) #find precise point in profile
             #find new max height
             if gaim_ne>max_ne:
                 max_ne    =gaim_ne
                 max_height=alt
             output+=L.strip()+" %5.5e %5.5e\n"%(gaim_ne,gaim_ne-n)
         out=open(f.replace(".profile",".profile2"),"w")
         out.write(make_header("profile2","gaim.profile",oldheader+";GAIM_HMF2:%05.1f,GAIM_NMF2:%05.5e"%(max_height,max_ne))+"\n")
         out.write(output)

if __name__ == "__main__":
  main()

