#!/usr/bin/perl -w
#######################
# Lukas Mandrake
#######################
# Provides the standard include values used in GAIM to a perl script
#######################

use strict;

package GAIM_Constants;

sub new
{
 my $c_me       = 9.1093818872e-31;
 my $c_e0       = 8.854187817e-12;
 my $c_e        = 1.60217646263e-19;
 my $c_pi       = 3.14159265358979323846264338328;
 my $c_aearth   = 6378.1363;
 my $c_fearth   = 1.0 / 298.257;
 my $c_mass_o   = 2.656864e-26;
 my $c_g        = 6.6725985e-11;
 my $c_m_earth  = 5.9737e24;

 my %constants = (

# Fundamental Constants

 "c"         => 2999792458                     ,      # speed of light (m/s)
 "pi"        => $c_pi                          ,      # famous Pi (not Xiaoqing)
 "boltzk"    => 1.38065812e-23                 ,      # Boltzmann constant (J/K)
 "e"         => $c_e                           ,      # elementary electron charge (C)
 "e0"        => $c_e0                          ,      # Permittivity of Free Space (F/m)
 "me"        => $c_me                          ,      # mass of electron (kg)
 "mp"        => 1.672623110e-27                ,      # mass of proton (kg)
 "mass_o"    => $c_mass_o                      ,      # mass of atomic Oxygen (kg)
 "mass_n2"   => 4.649512e-26                   ,      # mass of molecular Nitrogen (kg)
 "mass_o2"   => 5.313728e-26                   ,      # mass of molecular Oxygen (kg)
 "g"         => $c_g                           ,      # Gravitational Constant (m3/kg/s2)
 "avogadro"  => 6.022136736e23                 ,      # Avogadro's constant (1/mole)

# GPS-related Constants

 "l1_freq"   => 1575.42e6                      ,      # GPS L1 frequency (MHz)
 "l2_freq"   => 1227.60e6                      ,      # GPS L2 frequency (MHz)

# Earth-related constants

 "m_earth"   => $c_m_earth                     ,      # Mass of the Earth (kg) 1/28/91 value from Goddard SLR# "b0"        => 0.3e-4                         ,      # dipole moment (tesla) at the equator
 "r_earth"   => 6370.0e3                       ,      # earth radius, km
 "r_equator" => 6378.136e3                     ,      # earth radius at equator, km
 "fearth"    => $c_fearth                      ,      # Earth geoid f_earth constant
 "aearth"    => $c_aearth                      ,      # Earth geoid a_earth constant
 "j2"        => 1.0827e-3                      ,      # J2 gravity term
 "g_earth"   => 9.80665                        ,      # standard gravitational acceleration at sea level (m/s2)

# Conversions

 "phase2tec" => 9.517706325                    ,      # conversion factor from l1-l2 to TEC
 "recom_rate_1356" => 7.8e-19                  ,      # recombination rate for EUV (m^-3/s)

# These requier special treatment, as they are functions of the above

 "fof2_to_ne"=> $c_me*$c_e0/$c_e/$c_e*2*$c_pi*2*$c_pi, # conversion foF2 to density (s2/m3)
	         				       # comes out ~1/80.6, 2*pi due to omega vs f
 "bearth"    => $c_aearth * (1 - $c_fearth)    ,       # Earth geoid b_earth constant
 "d2r"       => $c_pi/180                      ,       # degree to radian 
 "r2d"       => 180/$c_pi                      ,       # radian to degree
 "mass_oi"   => $c_mass_o-$c_me                ,       # mass of atomic Oxygen ion (kg)
 "u0"        => 4.0*$c_pi*1.0e-7               ,       # Permeability of Free Space (N/A2)
 "gm"        => $c_g*$c_m_earth                        # Gravitation_const * earth mass
                                               );
 return bless (\%constants,shift);
}

sub display
 {
  my $self = shift;
  my $key = lc(shift);
  return $$self{$key};
 }



1;
