#!/usr/local/bin/python
#
# ionoutil.py -- Globally useful routines for input/output, file access,
#                shell manipulation, and UNIX command replacements
#                in a Pythonic sense.
#
# Lukas Mandrake 3/23/06

import sys, os, types, time, tempfile, re, pwd,random,thread
from math import *
from ionotime import *
from glob import *
#import numpy as N
import Numeric as N   # bdw, change to Numeric for sparc port

QUIET = 1 #flag indicating that these routines should not be so noisy

def echo   (str          ): sys.stdout.write(str + "\n")
def err    (str          ): sys.stderr.write(str + "\n")
def warn   (str          ): err("---WARNING: "+str)
def die    (str, status=1): err("***ERROR: "+str); sys.exit(status)
def stop   ()             : sys.exit(0)
def  ex    (command      ): return "".join(os.popen(command).readlines())
#execute and return lines together
def  exs   (command      ): os.popen(command) #execute without returning any information

def ext    (command      ):
   print "--->",command
   id=os.fork()
   if id is 0:
      os.system(command)
      sys.exit()
   return id
#   return os.spawnvp(os.P_NOWAIT,command.split()[0],command.split())
#   return thread.start_new_thread(ex,(command,)) #execute in a seperate thread


def exa    (command      ): return os.popen(command).readlines()          #execute and return array of results
def eex    (command      ):
   print command
   return ex(command)       #print command, then ex
def eextime(command      ):                                               #do eex but also print timing information
    begintime = time.time()
    answer=       "Begin at "+time.asctime()+"\n"
    answer=answer+eex(command)
    endtime   = time.time()
    answer=answer+"End   at "+time.asctime()+"\n"
    answer=answer+"Total runtime %0.1f minutes"%((endtime-begintime)/60.0)
    return answer
def safemax(list):
   if len(list)<1: return None
   return max(list)
def safemin(list):
   if len(list)<1: return None
   return min(list)

#grab any arrangement of indeces from an array at the same time
#PERL style, don't know why they don't support this natively
def index(list,indeces): return [list[x] for x in indeces]

#quickly returns a list of matching groups from a regular expression
#in a single function call. Failure to match returns [].
#Success returns [1,2,3] filled with matching fields
def regmatch(patternstring,tomatch):
   pattern=re.compile(patternstring)
   match=pattern.search(tomatch)
   if not match: return []
   else        : return match.groups()

#cycles through a list of dirs and
#expands them all to absolute while also
#validating their existence
def makefull_and_validate_dirs(list):
   curpath=os.getcwd()
   res=[]
   for x in list:
      os.chdir(x)
      res.append(os.getcwd())
      os.chdir(curpath)
   return res

#makes a series of numbers safe to take the log of
#sets all numbers 0 or less to least positive number found
def logsafe(list):
   #first find lowest still positive number
   positive_numbers=[x for x in list if x>0]
   if not positive_numbers: lowestpos=0.1 #default value if everything is negative or zero
   else                   : lowestpos=min(positive_numbers)
   return [max(lowestpos,x) for x in list]

#fit line to (x0,y0) and (x1,y1) to get xd
def linear_fit(x0,y0,x1,y1,xd):
   (x0,y0,x1,y1,xd)=[float(x) for x in (x0,y0,x1,y1,xd)]
   if abs(x0-x1) > 0: #check that there is a slope
       m=(y0-y1)/(x0-x1)
       b=y0-x0*m
       return m*xd+b
   else: #first two datapoint x's are identical, so just return their y avg
       return (y0+y1)/2.0

#reverses a list, even if it was a tuple
def reversed(series):
     work=list(series)
     work.reverse()
     return work

#walks through a list of pairs, first being index, finds max value in 2nd column, and returns pair
def find_pair_max(pair_list): return reversed(max([reversed(x) for x in pair_list]))

#Given an array (list of pairs), will take an index and
#match it to the first column of each pair, find the
#proper mix, and return the appropriate interpolated
#results from the 2nd column
#optional argument outside specifies a single value to use outside the grid, otherwise extrapolates
def interpolate_pair_array(pair_list,index,outside=None):
    #if we have an empty list, return nothing
    if not pair_list: return None
    #if we only have 1 point, return it
    if len(pair_list) < 2: return pair_list[0][1]
    #sort the list by index for efficient scanning
    newlist=sorted(pair_list)
    #if index <= whole list, extrapolate
    if index <= newlist[ 0][0]:
       if not outside == None: return outside
       return linear_fit(newlist[0][0],newlist[0][1],newlist[1][0],newlist[1][1],index)
    #if index >= whole list, extrapolate    
    if index >= newlist[-1][0]:
       if not outside == None: return outside
       return linear_fit(newlist[-1][0],newlist[-1][1],newlist[-2][0],newlist[-2][1],index)
    #requested index is within the list range, so interpolate
    t=0
    while newlist[t][0]<index: t+=1 #walk list until we find the point where index is exceeded... this is the right hand value
    #do simple linear interpolation between the two points
    return linear_fit(newlist[t-1][0],newlist[t-1][1],newlist[t][0],newlist[t][1],index)

#makes all possible combinations of -single, --single, -full, --full switches
def switchvariants(single,full):
    return ("-"+single,"--"+single,"-"+full,"--"+full)

def compareto (num1,num2,power): #compare to the listed power. 0 is 1's, 2 is 100's, -2 is 0.01's, etc.
   if not (isinstance(num1,types.ListType) or isinstance(num1,types.TupleType)): num1=(num1,)
   if not (isinstance(num2,types.ListType) or isinstance(num2,types.TupleType)): num2=(num2,)
   if not len(num1) == len(num2): return False
   ans=True
   for t in range(len(num1)):
      ans=ans and int(num1[t]/10.0**power+0.5) == int(num2[t]/10.0**power+0.5)
   return ans

def wrap_number(x,lower=0,upper=360): #normalizes x to be between lower and upper, wrapping if needed
    #lower bound is inclusive, upper exclusive
    u=max(lower,upper)
    d=min(lower,upper)
    delta=u-d
    while x <  d: x+=delta
    while x >= u: x-=delta
    return x

def angle_compare(x,y,BASE=360): #average in a degree sense (circle), or set BASE to something else for say time (24)
      x=wrap_number(float(x),0,BASE)
      y=wrap_number(float(y),0,BASE)
      d=abs(y-x)
      if d<=BASE/2.0: return ((x+y)/2.0,d) #average and difference, easy if angles aren't near 0-360 interchange
      x=wrap_number(x,-BASE/2.0,BASE/2.0)
      y=wrap_number(y,-BASE/2.0,BASE/2.0)
      d=abs(y-x)
      a=wrap_number((x+y)/2.0,0,BASE)
      return (a,d)
     
#def mean   (lister): return N.array([float(x) for x in lister]).mean()
# Rewrite mean function for Numeric, for sparc port
def mean   (lister):
   a = N.array([float(x) for x in lister])
   return N.sum(a)/float(len(a))

#def sum    (lister): return N.array([float(x) for x in lister]).sum()
# Rewrite mean function for Numeric, for sparc port
def sum    (lister): return N.sum(N.array([float(x) for x in lister]))

def std    (lister): return N.array([float(x) for x in lister]).std()
def rms    (lister):
   L=N.array([float(x) for x in lister])
   return pow(L.std(),2)+pow(L.mean(),2)

#properly generate a header string
def make_header(extension,generator="unspecified",purpose="unspecified"):
    extension=extension.lower()
    header=''.join(open(os.environ.get('GAIM')+"/scripts/header."+extension).readlines())
    header=header.replace("${date}",IonoTime().now().to("YYYYMMDD_HHMMSS"))
    header=header.replace("${generator}",generator)
    header=header.replace("${purpose}",purpose)
    return header.strip()

def safe_cat(destination,*list): #concatenate all these files (even if gzipped) no matter if the glob would overflow
    delfile(destination,quiet=True)
    for x in list:
        for f in glob(x):
            ex("icat "+f+" >> "+destination)

#ensures a directory isn't changing for s seconds
def ensure_unchanging_directory(dir,s):
    d=getcompletedir(dir)
    flag=True
    while True:
        time.sleep(s)
        n=getcompletedir(dir)
        if n==d:
            break
        else:
            d=n

#returns all expanded full paths
def getfulldir(path):
    if not isinstance(path,types.ListType): path=[path]
    ret=[]
    for x in path:
        if os.path.isdir(x): x+="/*"
        ret+=[os.path.abspath(y) for y in glob(x)]
    return ret

#returns the system complete absolute path.. by convention, path must end in /
def getdir(path):
    return [os.path.split(y)[1] for y in getfulldir(path)]

#returns a full directory listing of a specified directory
#structures contain [filename,date,owner,size]
def getcompletedir(path,ignorelinks=False,ignorenames=False):
    retlist=[]
    files=getfulldir(path)
    for x in files:
        if ignorelinks and os.path.islink(x): continue #skip links if directed to do so
        if os.path.isdir(x):
            retlist.extend(getcompletedir(x,ignorelinks,ignorenames))
            continue
        (mode,ino,dev,nlink,uid,gid,size,atime,mtime,ctime)=os.stat(x)
        try:
            o=pwd.getpwuid(uid)[0]
        except:
            o="???"
        if ignorenames: x=None
        retlist.append([x,IonoTime(mtime,'J1970'),o,size])
    return retlist

#searches the target string for any of a list of strings in the following list
def multisearch(target,list):
    if isinstance(list,types.StringType): searchlist=[list]
    else                                : searchlist=list
    for x in searchlist:
        if target.find(x)>-1: return True
    return False

def testfile(path        ): return os.path.exists(path)

def delfile(path         ,quiet=0):
    if not isinstance(path,types.ListType): path=[path]
    for x in path:
        if not quiet: print "-removing ",x
        os.popen("rm -f "+x)
        if (os.path.exists(x)): die("Failed to remove "+x)

def killdir (path        ,quiet=0):
     if not quiet: print "-removing ",path
     os.popen("rm -fr "+path)
     if (os.path.exists(path)):
          die("Failed to remove "+path)

def makedir (path        ,quiet=0):
     if not quiet: print "-making   ",path
     os.popen("mkdir -p "+path)
     if (not os.path.exists(path)):
          die("Failed to make "+path)

def linkfile (path1,path2,quiet=0):
     confirm(path1,quiet)
     if not quiet: print "-linking  ",path1,"to",path2
     ex("rm -f "+path2)
     os.popen("ln -s "+path1+" "+path2)
     confirm(path2,quiet)

def filetimeutc(path,quiet=0): #return utc timetag of a file's last modification date
     confirm(path,quiet)
     return IonoTime(convert_1970_to_J2000_epoch(os.stat(path).st_mtime))

def filetime(path,quiet=0): #return local time timetag of a file's last modification date
    localtimedelta=IonoTime().now()-IonoTime().nowUTC() #num seconds between UTC and localtime
    return filetimeutc(path,quiet).addSeconds(localtimedelta)

#link first file found matching to somewhere else
def linkfileany (path1,path2,quiet=0):
     #find wildcard matches before linking... but only select one (first)
     arr=getfulldir(path1)
#     if len(arr)<1: die ("no wildcard match for linkfileany: "+path1)
     first=arr[0].strip()
     if not quiet: print "-linking  ",path1,"match",first,"to",path2
     os.popen("ln -s "+first+" "+path2)
     confirm(path2)

#link all files found in one place to another place
def linkfileall (path1,path2,quiet=0):
     #find wildcard matches and link them all to path2 which must NOT be a file
     arr=getfulldir(path1)
#     if len(arr)<1: die ("no wildcard match for linkfileall: "+path1)
     for x in arr:
          x=x.strip()
          linkfile(x,path2,quiet)

#link all files found in a place by wildcard to PWD
def linkfileallhere (path1,quiet=0):
     #find wildcard matches and link them current directory 
     arr=getfulldir(path1)
#     if len(arr)<1: die ("no wildcard match for linkfileall: "+path1)
     for x in arr:
          x=x.strip()
          linkhere(x,quiet)

#copy a file from one place to another
def copyfile (path1,path2,quiet=0,dereference=0):
     confirm(path1)
     if not quiet: print "-copying  ",path1,"to",path2
     deref=""
     if dereference: deref="L"
     os.popen("cp -fr"+deref+" "+path1+" "+path2)
     confirm(path2,quiet)

#dump contents to specified path as a file
def writefile(path,contents):
     ex("rm -f "+path)
     writer=file(path,"w")
     writer.write(contents)
     writer.close()
     confirm(path)

def copyfilewithreplacement(path1,path2,substitutions,quiet=0):
     confirm(path1,quiet)
     if not quiet: print "-copying  ",path1,"to",path2,"with subs"
     wholefile=ex("cat "+path1)
     for x in substitutions.keys():
          wholefile=wholefile.replace(x,substitutions[x])
     writefile(path2,wholefile)

def movefile (path1,path2,quiet=0):
     if path1 == path2: return
     confirm(path1,quiet)
     if not quiet: print "-moving   ",path1,"to",path2
     os.popen("mv -f "+path1+" "+path2)
     confirm(path2,quiet)

def linkhere (path1,quiet=0):
    if not isinstance(path1,types.ListType): searchpath=[path1]
    else                                   : searchpath= path1
    for x in searchpath:
        if not quiet: print "-linking  ",x,"to <cwd>"
        os.popen("ln -s "+x+" .")

def confirm (path        ,quiet=0):
     if not quiet: print "-checking",path
     if (not os.path.exists(path)): die("Necessary file not present "+path)
     return True

def confirm_nonzero (path        ,quiet=0):
     confirm(path,quiet)
     if (os.path.getsize(path) < 1): die("Necessary file zero length "+path)
     return True

def confirm_all (path_pattern        ,quiet=0):
     files = glob(path_pattern)
     if (len(files) == 0): die("No file matching the pattern present " + path_pattern)
     return True

def confirm_nonzero_all (path_pattern        ,quiet=0):
     files = glob(path_pattern)
     if (len(files) == 0): die("No file matching the pattern present " + path_pattern)
     for f in files:
          confirm_nonzero(f, quiet)
     return True
     
def isnumber (teststring):
     if (teststring.find(",") > -1) : return False
     teststring=teststring.strip(" \n")
     noexponent=teststring.replace("d","")
     noexponent=noexponent.replace("e","")
     noexponent=noexponent.replace(".","")
     return noexponent.isdigit()

def expandfullpath (file):
     """expands a relative file path to the full absolute one"""
     return os.path.abspath(file)

#Take the elevation (in degrees) and return the mapping function
def mapping_function(el):
    RADIUS_EARTH             = 6370.0 #km
    IONOSPHERIC_SHELL_HEIGHT = 450.0  #km
    PI                       = 3.141592654
    t=cos(el/180.0*PI)/(1+IONOSPHERIC_SHELL_HEIGHT/RADIUS_EARTH)
    return 1.0/sqrt(1.0-t*t)

def sendmail(users,subject,message):
    if not isinstance(users,types.ListType): users=[users]
    t=tempfile.NamedTemporaryFile()
    t.write(message)
    t.flush()
    for x in users: ex("mail "+x.strip()+" -s '"+subject.strip()+"' < "+t.name)

#returns strings older than a YYYYMMDD_HHMMSS or YYMMDD_HHMMSS internal tag anywhere in the string
def youngerthan(timetag,list):    
    k=[]
    t=IonoTime(timetag)
    filt1=re.compile(".*(\d\d\d\d\d\d\d\d_\d\d\d\d\d\d).*")
    filt2=re.compile(".*(\d\d\d\d\d\d_\d\d\d\d\d\d).*")
    for x in list:
        match1=filt1.match(x)
        if match1:
           if IonoTime(match1.groups()[0]) > t: k.append(x)
        else:
            match2=filt2.match(x)
            if match2:
                if IonoTime(match2.groups()[0]) > t: k.append(x)
    return k

def olderthan(timetag,list):    
    k=[]
    t=IonoTime(timetag)
    filts=[[re.compile(".*(\d\d\d\d\d\d\d\d_\d\d\d\d\d\d).*"),"YYYYMMDD_HHMMSS"],
           [re.compile(".*(\d\d\d\d\d\d_\d\d\d\d\d\d).*")    ,"YYMMDD_HHMMSS"  ],
           [re.compile(".*(....\d\d\d\D\d\d\.\d\d).*")       ,"TENETHOURLY"    ] ]
           
    for x in list:
        for f in filts:
            filt,format=f
            match=filt.match(x)
            if match:
                if IonoTime(match.groups()[0],format) < t: k.append(x)
    return k

def removeolderfiles  (timetag,path,quiet=0): delfile(  olderthan(timetag,getfulldir(path)),quiet)
def removeyoungerfiles(timetag,path,quiet=0): delfile(youngerthan(timetag,getfulldir(path)),quiet)

#returns list in which all elements are unique
def unique(list):
    a={}
    for x in list: a[x]=1
    return a.keys()

#provides list access to an array or list
def g(obj,list):
    ret=[]
    for x in list:
        ret+=[obj[x]]
    return ret

#create a file quickly of given name and contents
def makefile(name,contents):
    w=open(name,"w")
    w.write(contents)
    w.close()
    
###translate between svn and prn easily
class gpssatnum:
    def __init__(self):
        self.svntoprn={}
        self.prntosvn={}
#        print "nrt2dump sees GAIM as"+os.environ['GAIM']
        svnfile=open(os.environ.get('GAIM')+'/inputs/svnprn.tbl')
        for line in svnfile:
            if line[0]=="#": continue
            (svn,prn)=line.split()[0:2]
            self.svntoprn[float(svn)]=float(prn)
            self.prntosvn[float(prn)]=float(svn)

    def svn2prn(self,svn):
        return self.svntoprn[float(svn)]

    def prn2svn(self,prn):
        return self.prntosvn[float(prn)]

###Makes a column-oriented text file lined up evenly on boundaries for prettiness
###Takes an array of simple strings about to be printed, and fixes 'em
def clean_columns(outarray,ignore_headers=False):
   if len(outarray)<2: return outarray
   #first obtain the statistics on all the lines
   maxwidths=[0]*1000 #max columns to handle is 1000 for zip function below
   header=[]
   for line in outarray:
      if line[0]=="#" and ignore_headers: #do not include headers in resizing, just pass them through gathered at the top
         header.append(line.strip()+"\n")
         continue
      colwidths=[len(x) for x in line.strip().split()]
      maxwidths=[max(x,y) for x,y in zip(colwidths,maxwidths)]
   #now reformat each line with each column appropriately spaced
   #do so by making a custom format string
   format="".join(["%%%0ds "%(w) for w in maxwidths])[:-1]+"\n"
   format=format.replace("%","%-",1) #make only first instance have left justification for ease of reading (and ensuring slot [0] is always header #)
#   print format
#   print len(outarray[0].strip().split())
#   print outarray[0].strip().split()
   if ignore_headers: return header+[format%tuple(x.strip().split()) for x in outarray if not x[0]=="#"]
   else             : return        [format%tuple(x.strip().split()) for x in outarray                 ]

##this is the opposite of zip... it makes a list of [(1,2),(3,4),(5,6)] into [(1,3,5),(2,4,6)]
##I call it unzip
def unzip(list): return [ [x[t] for x in list] for t in range(len(list[0]))]

###Running average on a list of data given
###a window size to scroll
###Setting can make it wrap or not (cyclical)
def running_average(list,window,wrap=False):
   if window < 1 : return [] #undefined, must sample positive integer number of cells
   L=len(list)
   list=N.array([float(x) for x in list])
   if window >= L:
      if not wrap : return [list.mean(),] #just a single value for too large a window
      else        : return [list.mean(),]*L
   #handled large window cases, now handle midsections
   if not wrap    : return [               list[t:t+window]                                   .mean() for t in range(L-window+1)] #return value length = len(list)-window+1
   else           : return [N.concatenate((list[t:min(t+window,L)],list[0:max(0,t+window-L)])).mean() for t in range(L         )] #return value length = len(list)

###evaluates polynomial defined by coefs at value x ... redundant with N.polyval
def polyeval(x,coefs,xscale=None,yscale=None):
   if not xscale is None and yscale is None: die("polyeval must either have neither xscale and yscale, or both")
   coefs=reversed(coefs)
   if xscale is None: return sum([pow(x,order)*coefs[order]                          for order in range(0,len(coefs))])
   else             : return sum([pow(x,order)*coefs[order]*yscale/pow(xscale,order) for order in range(0,len(coefs))])

###Extends and fills gaps in a series of data using
###an Nth order polynomial as per polyfit
###order is order of poly to use, begin is beginx, end is endx, numpts is number of points in total to use
def polyfit_series(xvals,yvals,order=1,begin=None,end=None,numpts=None):
   if begin  is None: begin =xvals[ 0]
   if end    is None: end   =xvals[-1]
   if numpts is None: numpts=len(xvals)

   if order > len(xvals)-1: return (xvals,yvals,0.0) #if requested order fit is more than data, is underdetermined

   #scale data... unscale afterwards. this prevents roundoff error
   scaledx=N.array([float(x) for x in xvals])
   xfactor=max(scaledx)-min(scaledx)
#   xfactor=1.0
   scaledx/=xfactor

   scaledy=N.array([float(y) for y in yvals])
   yfactor=max(scaledy)-min(scaledy)
#   yfactor=1.0
   scaledy/=yfactor

#   print xfactor,yfactor

#   print scaledx
#   print scaledy
#   print order
   coefs=N.polyfit(scaledx,scaledy,order)

#   print reversed(coefs)
#   print [reversed(coefs)[order]/yfactor*pow(xfactor,order) for order in range(0,len(coefs))]

   delta=(float(end)-float(begin))/float(numpts-1)

   newx  =N.array([x for x in N.array(range(0,numpts))*delta+begin ])
   newy  =N.array([polyeval(x,coefs,xfactor,yfactor) for x in newx ])
   checky=N.array([polyeval(x,coefs,xfactor,yfactor) for x in xvals])
   resids=sqrt(sum([pow(x-y,2) for x,y in zip(yvals,newy)]))

   return (newx,newy,resids)

#autoscales the order of polynomial fit to find best fit and uses that
def polyfit_autoorder(xvals,yvals,begin=None,end=None,numpts=None,beginorder=0,endorder=None):
   if endorder is None: endorder=min(len(xvals)/2,20) #don't go beyond 20 unless user specifies
   beginorder=int(beginorder)
   endorder  =int(endorder)
   minresid=99e99
   minorder=0
   rx=ry=[]
   for order in [o+beginorder for o in range(0,endorder-beginorder)]:
      (x,y,resid)=polyfit_series(xvals,yvals,order,begin,end,numpts)
      if resid<minresid: (minresid,rx,ry,minorder)=(resid,x,y,order)
   return (rx,ry,minorder,minresid)

#write out a numpy array to a CSV file
def write_2d_array(filename,array):
   f=open(filename,"w")
   array=N.array(array)
   (mx,my)=array.shape
   for x in range(mx):
      f.write(",".join(["%f"%(z) for z in array[x,:]])+"\n")
   f.close()

def write_1d_array(filename,array): open(filename,"w").write(",".join(["%f"%z for z in list(array)])+"\n")

#explores the given path and removes any files that are found to be links
def remove_links(path):
   for f in getdir(path):
      if os.path.islink(f): ex("rm "+f)

def main():
   print "Checking Utils"

if __name__ == "__main__":
  main()


