#!/bin/bash 

#source /iono2/gaim/v2k/source/env/Linux86/opt/gaim.env.sh

LIVEDIR=$GAIM/ops/hourly/live/grids
POSTFIX=""

usage="Usage: $(basename $0) [-l dir] [-p postfix]
    dir:      directory where the matlab state files are.
              Default: $GAIM/ops/hourly/live/grids
    postfix:  append this postfix to the file, when copying into the
              public directory.
              Default: \"$POSTFIX\"
              Use \"today\", and postfix will be _daily_YYMMDD, 
              where YYMMDD is today's date UT.
"

while getopts hl:p: c; do
    case $c in
	l) LIVEDIR="$OPTARG";;
	p) POSTFIX="$OPTARG";;
	*) echo "$usage"; exit 0;;
    esac
done

LOG=$LIVEDIR/nrt_gaim.make_jpeg.log

UT_DATE=$(date -u +%y%m%d)

if [ "$POSTFIX" == "today" ]; then
    POSTFIX="_daily_$UT_DATE"
fi

cd $LIVEDIR

# use a lock file so we don't start making a movie with an 
# incomplete frame (the same lockfile will be used in the movie 
# generating script)
lockfile -30 -r4 movie.lock
if [ $? -ne 0 ]; then
    echo "nrt_gaim.make_jpeg.sh: Failed to get lock file." >> $LOG
    exit 1
fi

date >> $LOG
echo "making 3 panel and tec jpegs for $UT_DATE UT" >> $LOG

if [ ! -f jpeg.mk ]; then
    echo "%.mat.3panel.jpg: %.mat" > jpeg.mk
    echo "	vtec_and_dens_slice.pl \$<" >> jpeg.mk
    echo >> jpeg.mk
    echo "%.mat.tec.jpg: %.mat" >> jpeg.mk
    echo "	vtec_and_dens_slice.pl \$<">> jpeg.mk
fi

##for i in kalman_*[0-9]*_image.mat; do
/bin/ls -1 kalman_*[0-9]*_image.mat | sort | while read i; do
    make -f jpeg.mk $i.3panel.jpg 1>>$LOG 2>&1
    echo $i
done

latest=$(/bin/ls -1 kalman_*[0-9]*_image.mat.3panel.jpg | sort | tail -1)
ln -sf $latest kalman_assim_latest_3panel.jpg
latest=$(/bin/ls -1 kalman_*[0-9]*_image.mat.tec.jpg | sort | tail -1)
ln -sf $latest kalman_assim_latest_tec.jpg

echo "Copying the latest jpgs to the web directory" >> $LOG
cp kalman_assim_latest_3panel.jpg /ecomm/iono/web/gaim/rtdata/kalman_assim_latest_3panel$POSTFIX.jpg 2>> $LOG
cp kalman_assim_latest_tec.jpg /ecomm/iono/web/gaim/rtdata/kalman_assim_latest_tec$POSTFIX.jpg 2>> $LOG

echo "Done. $(date)" >> $LOG
echo "--------------------------------------------" >> $LOG
echo >> $LOG

# release the lock
echo "rm -f movie.lock" >> $LOG
rm -f movie.lock
