#!/bin/bash 

#source /iono2/gaim/v2k/source/env/Linux86/opt/gaim.env.sh

LIVEDIR=$GAIM/ops/hourly/live/grids
DECIMATE=15m
POSTFIX=""

usage="Usage: $(basename $0) [-l dir] [-d decimate] [-p postfix]
    dir:      directory where the matlab state files are.
              Default: $LIVEDIR
    decimate: time delta for the jpeg frames to be used int he movie.
              Default: $DECIMATE
    postfix:  append this postfix to the file, when copying into the
              public directory.
              Default: \"$POSTFIX\"
              Use \"today\", and postfix will be _daily_YYMMDD,
              where YYMMDD is today's date UT.
"

while getopts hl:d:p: c; do
    case $c in
	l) LIVEDIR="$OPTARG";;
	d) DECIMATE="$OPTARG";;
	p) POSTFIX="$OPTARG";;
	*) echo "$usage"; exit 0;;
    esac
done


LOG=$LIVEDIR/nrt_gaim.make_movie.log

UT_DATE=$(date -u +%y%m%d)

if [ "$POSTFIX" == "today" ]; then
    POSTFIX="_daily_$UT_DATE"
fi

cd $LIVEDIR

# use a lock file so we don't start making a movie with an 
# incomplete frame (the same lockfile will be used in the frame 
# generating script)
lockfile -60 -r8 movie.lock
if [ $? -ne 0 ]; then
    echo "nrt_gaim.make_movie.sh: Failed to get lock file." >> $LOG
    exit 1
fi

date >> $LOG
echo "making 3 panel and tec movies for $UT_DATE UT" >> $LOG

# save the previous results
cp -p used_in_movie used_in_movie.previous 2>/dev/null
cp -p 3panel.mpg 3panel_previous.mpg 2>/dev/null
cp -p tec.mpg tec_previous.mpg 2>/dev/null

# get the list of frames to use
$GAIM/scripts/nrt_gaim.generate_kalman_log $UT_DATE $LIVEDIR $DECIMATE 2>used_in_movie.err |
sed 's/  */\t/g;s/\.mat/_image.mat/g' |
cut -f5 > used_in_movie 

if [ "$(cat used_in_movie.err)" != "" ]; then
    # nrt_gaim.generate_kalman_log resulted in an error
    # use all the image files
    /bin/ls -1 $LIVEDIR/kalman*image.mat > used_in_movie
fi

lst=$(cat used_in_movie)
lst_3panel=$(for i in $lst; do if [ -f $i.3panel.jpg ]; then echo -n "$i.3panel.jpg "; fi; done)
lst_tec=$(for i in $lst; do if [ -f $i.tec.jpg ]; then echo -n "$i.tec.jpg "; fi; done)

if [ "$lst_3panel" != "" ]; then
    echo "3panel.mpg: $lst_3panel" > 3panel.mk
    echo "	convert \$+ \$@" >> 3panel.mk
    make -f 3panel.mk 3panel.mpg  1>>$LOG 2>&1
    cp 3panel.mpg /ecomm/iono/web/gaim/rtdata/3panel$POSTFIX.mpg 2>> $LOG
else
    echo "No jpegs to create the 3panel movie" >> $LOG
fi
echo "Done 3panel. $(date)" >> $LOG
echo "++++++++++++++++" >> $LOG

if [ "$lst_tec" != "" ]; then
    echo "tec.mpg: $lst_tec" > tec.mk
    echo "	convert \$+ \$@" >> tec.mk
    make -f tec.mk tec.mpg 1>>$LOG 2>&1
    cp tec.mpg /ecomm/iono/web/gaim/rtdata/tec$POSTFIX.mpg 2>> $LOG
else
    echo "No jpegs to create the tec movie" >> $LOG
fi

echo "Done. $(date)" >> $LOG
echo "--------------------------------------------" >> $LOG
echo >> $LOG

# release the lock
rm -f movie.lock
