#!/usr/bin/perl -w 
#######################
# Adapted by Vardan Akopian from original by Lukas Mandrake
#######################

use strict;

#############################################################################
#MAIN BODY
#############################################################################

my ($help_flag, $image_file, $vmin, $vmax, $title, $xlabel, $ylabel, $units, $outfile);
my ($proj, $colorbar, $xsize, $ysize, $dpi, $format, $matlab);

#$matlab = "/usr/local/matlab/bin/matlab";
$matlab = "matlab";

# the location of ImagMagick's convert program
# for unix
my $convert = "convert";
# for windows
#my $convert = "/usr/local/ImageMagick/bin/convert";

if ($#ARGV<0) {
  $help_flag=1;
} else { 
  $help_flag=0;
  $image_file = shift;
  while (@ARGV) {
    my $arg = shift;
    if    ($arg =~ /^-h/     ) { $help_flag=1; last; }
    elsif ($arg =~ /^-vmin/  ) { $vmin    = shift; }
    elsif ($arg =~ /^-vmax/  ) { $vmax    = shift; }
    elsif ($arg =~ /^-t/     ) { $title   = shift; }
    elsif ($arg =~ /^-xl/    ) { $xlabel  = shift; }
    elsif ($arg =~ /^-yl/    ) { $ylabel  = shift; }
    elsif ($arg =~ /^-units/ ) { $units   = shift; }
    elsif ($arg =~ /^-out/   ) { $outfile = shift; }
    elsif ($arg =~ /^-proj/  ) { $proj    = shift; }
    elsif ($arg =~ /^-x/     ) { $xsize   = shift; }
    elsif ($arg =~ /^-y/     ) { $ysize   = shift; }
    elsif ($arg =~ /^-format/) { $format  = shift; }
    elsif ($arg =~ /^-bar/   ) { $colorbar= shift; }
    elsif ($arg =~ /^-dpi/   ) { $dpi     = shift; }
    else {print "nv_fltr_3d_tec: Fatal error, bad command line argument: $arg\n"; $help_flag=1; last;}
  }
}

if (!$image_file || $image_file =~ /^-/ ) {
  $help_flag = 1;
} elsif (! -f $image_file) {
  die "no such file $image_file";
}

if ($help_flag) {
  print <<EOH
Usage: nv_flt_3d_tec <image_file> [switches]

Plots Vertical TEC from a image mat file. Output is generated into any
requesetd format type. The switches are:

-vmin  NUMBER	 . Default: 0

-vmax  NUMBER	 . Default: 100

-t     TITLE	 Title for each graph. Default: Vertical TEC

-xl    LABEL	 Label for x axis. Default: none

-yl    LABEL     Label for y axis. Default: none

-units LABEL	 Label for color scale. Default: TECU

-x     PIXELS	 Pixel size of desired image. Default: no resize

-y     PIXELS	 Pixel size of desired image. Default: no resize

-bar   TYPE	 Colorbar type. Valid responses: horiz, vert, or none.
		 Default: vert.

-format TYPE	 Format of desired output file. Valid responses are TIF,
		 PS, JPG, PNG, PCX. Default: JPG

-out   FILE	 Final image file name. Default: <image_file>\.<format>

-proj  TYPE	 Projection type to use. Default MILLER. Valid settings are
		 MILLER, GLOBE, EQDAZIM, MOLLWEID, BRIES, BONNE, POLYCON,
		 EQACONIC, WINKEL, SINUSOID, ROBINSON, BALTHSRT, BEHRMANN,
		 BSAM, BRAUN, CASSINI, CCYLIN, EQACYLIN, EQDCYLIN, GISO,
		 GORTHO, GSTEREO, LAMBCYLN, MERCATOR, PCARREE, TRYSTAN,
		 UTM, WETCH, and APIANUS. Using projections other than
		 MILLER does not guarentee proper axis label placement.
-dpi   NUMBER	 Dot per inch resolution for the output file. Default: 300

-h         	 Print this help message.

Example: nv_flt_3d_tec kalman_XXX_image.mat 

EOH
  ;
  exit;
}

if (!$vmin) { $vmin = 0; }
if (!$vmax) { $vmax = 100; }
if (!$format) { $format = "jpg"; } else { $format = lc($format); }
if (!$outfile) { $outfile = $image_file.".".$format;}
if (!$proj) { $proj = "miller"; } else { $proj = lc($proj); }
if (!$xlabel) { $xlabel = ""; }
if (!$ylabel) { $ylabel = ""; }
if (!$xsize && $ysize) { die "resizing requires both x and y sizes"; }
if (!$ysize && $xsize) { die "resizing requires both x and y sizes"; }
if (!$colorbar) { $colorbar = "vert"; } else { $colorbar = lc($colorbar);}
if (!$title) { $title = "Vertical TEC"; }
if (!$units) { $units = "TECU";}
if (!$dpi) { $dpi = 300; }

#####YOU CANNOT GENERATE ANYTHING OTHER THAN PS FILES IN MATLAB NODISPLAY
my $workstring =
  "$matlab -nodisplay -nojvm -r \"addpath $ENV{'GAIM'}/visual/;";
#  "$matlab -nojvm -r \"";

$workstring.="overcome_matlab_license('";
#$workstring.="eval('";

$workstring.="nv_fltr_3d_tec(";
$workstring.="''$image_file'',$vmin,$vmax,1,''$proj'',''$title''";
$workstring.=",''$xlabel'',''$ylabel'',''$units'',''$colorbar'',$dpi)";
$workstring.="')";
$workstring.=";exit;\"";

my $resize;
if (!$xsize && !$ysize) {
  $resize = " ";
} else {
  $resize = " -resize $xsize"."x"."$ysize";
}
#Now we convert the Matlab PS files into whatever we're interested in
$workstring.=" && $convert $resize $image_file.ps $outfile && if [ \"$format\" != \"ps\" ]; then echo rm -f $image_file.ps; fi\n";
print "Running $workstring...\n\n";
print `$workstring`;
print "Done\n";
