#!/usr/bin/perl 
#
# stats.pl -- Simple statistics routine:  computes mean, sigma, min, max, rms.
#
# Example usage:
#
# Suppose @farr is an array of numbers, then the following lines compute
# and print the statistics:
#   @stats_out =  &stats(*farr);
#   &print_stats( "Label", *stats_out );
#
# The stats routine returns an array of ( $n, $mean, $sigma, $min, $max, $rms ).
# The print_stats routine prints the stats with a label.
#
# Note: The strange "*farr" syntax is required to pass an array into a
# subroutine.  Similarly, the @stats_out array is passed into the print_stats
# routine.

# Main routine for use as a filter.
@ary = <>;
@stats_out = &stats( *ary );
&print_stats( "<stdin>", *stats_out );


sub stats {
  local(*xs) = @_;
  local($n, $sum, $sumsq, $x, $mean, $sigma, $min, $max, $rms);

  $n = 0;
  $sum = 0;
  $sumsq = 0;
  foreach $x (@xs) {
    $n++;
    if ( $n == 1 ) {
      $min = $x;
      $max = $x;
    } else {
      $min = ( $x < $min ? $x : $min );
      $max = ( $x > $max ? $x : $max );
    }
    $sum += $x;
    $sumsq += $x * $x;
  }

  if ( $n > 0 ) {
    $mean = $sum / $n;
    $sigma = $sumsq - $n * $mean * $mean;
    if ( $n > 1 ) { $sigma = sqrt( $sigma/($n-1) ) };
    $rms = sqrt( $sumsq / $n );
  }

  ( $n, $mean, $sigma, $min, $max, $rms );
}

sub print_stats {
  local($name, *s) = @_;
  local($n, $mean, $sigma, $min, $max, $rms);

  print "\nStatistics for $name\n";
  print "N         Mean       Sigma         Min         Max         RMS\n";
  ($n, $mean, $sigma, $min, $max, $rms) = @s;
  printf "%d  %10.2f  %10.2f  %10.2f  %10.2f  %10.2f\n", 
      $n, $mean, $sigma, $min, $max, $rms;
}

1;
