#!/usr/bin/perl -w

#   str_replace.pl
#
#   Written  - 12/13/02, BDW
#--------------------------------------------------------------------------

use strict;
use lib "$ENV{'GAIM'}/perllib";
use Util;

use constant USAGE_MSG => <<EOS;

Usage: str_replace.pl -f <key/val pairs file>   files ...
       str_replace.pl -s <key/val pairs string> files ...

Purpose:  Do string replacements in one or more files, using a file or string
          containing key/val pairs.

Options:
    -f  File of key/val pairs separated by white space.
    -s  Multi-line string of key/val pairs.

EOS

my ($pairs_file, $pairs_str);
while ( ($_ = shift @ARGV) && m/^-/ ) {
    if ( m/^-f/ ) {
        $pairs_file = shift @ARGV;
    } elsif ( m/^-s/ ) {
        $pairs_str = shift @ARGV;
    }
}
my $file1 = $_;
unshift @ARGV, $_;

die USAGE_MSG
    unless ( (defined $pairs_file || defined $pairs_str) && defined $file1 );

my @lines;
if ( defined $pairs_file ) {
    open(IN, $pairs_file) || die "str_replace: Cannot open file, $pairs_file";
    @lines = <IN>;
    close(IN);
} else {
    @lines = split "\n", $pairs_str;
}

print STDERR "str_replace.pl: Performing following substitutions . . . \n";
my (%subs, $key, $val);
foreach my $line ( @lines ) {
    print STDERR $line;
    ($key, $val) = split ' ', $line;
    die "str_replace: Bad key/val pair: $line"
        unless defined $key && $key ne "" && defined $val && $val ne "";
    $subs{$key} = $val;
}
print STDERR "\nstr_replace.pl: On the files: " . join(" ", @ARGV) . "\n";

Util::assoc_str_replace(\%subs, @ARGV);

