#!/usr/bin/perl -w
# @(#)tec2gaim_azel.pl	1.1 6/16/00
# Edited by Lukas Mandrake, 12/08/03

use strict;
use lib "/iono/src/newgim/perllib";
use DateTime;

#$|=1;

if (! @ARGV)
{
 print "tec2gaim_azel.pl <gpstecYYMMDD.tec> <gaim_observation_file.tec_ref> <-d YYMMDD>

Converts a standard gim tec dump file into the GAIM observations tec_ref
format for input to a gaim run.

The optional -d switch provides an alternative date to use, regardless of filename.

";
}


my ($tecfile, $gaimfile, $opt, $dater) = @ARGV;
my ($shortdate, $opd);

if    (lc($opt)=~/^-d/) { $shortdate = $dater; }
elsif (! ($tecfile =~ /.*(\d\d\d\d\d\d).*/)) 
{ die "tecdumpfile must conform to nameYYMMDD.ext if you don't invoke -d\n"; }
else
{ $shortdate = $1; }

print "Latched date $shortdate => ";

my $dt = new DateTime;

$shortdate=~/(..)(..)(..)/; $dt->fromCalStr("20".$1."/".$2."/".$3.",00:00:00.00");

print $dt->toCalStr()," to ";

my $beginGPS = $dt->toSecPastTime("2000/01/01,00:00:00.00")-12*60*60;
               $dt->addDeltaTime("0/11:59:59.9");
my $endGPS   = $dt->toSecPastTime("2000/01/01,00:00:00.00");

print $dt->toCalStr(),"\n";

print "Processing GPStime $beginGPS to $endGPS\n";

#      $dt->fromSec($beginGPS);
#print $dt->toCalStr(),"\n";
#      $dt->fromSec($endGPS);
#print $dt->toCalStr(),"\n";

#exit

open(IN , $tecfile    );
open(OUT,"> $gaimfile");

my ($good_count, $bad_count) = (0,0);
my ($time, $rcv_id, $trn_id, $az, $el, $date_time, $stec, $rcv_x, $rcv_y, $rcv_z, $trn_x, $trn_y, $trn_z, $trn_bias, $rcv_bias);

my @tmp;
while (<IN>) {
    chomp;
    if (/^\#/) { next; }    ##remove header
    @tmp=split(' ',$_);
    if ( ($tmp[0] < $beginGPS) || ($tmp[0] > $endGPS) )
       {
	$bad_count++;
        next;
       }
    if ($good_count++ % 10000 == 0) { print "."; }
    $time     =$tmp[ 0]-$beginGPS;
    $rcv_id   =$tmp[ 1];
    $trn_id   =$tmp[ 2];
    $az       =$tmp[ 3];
    $el       =$tmp[ 4];
    $date_time=$tmp[ 5];
    $stec     =$tmp[ 6]*1.e16;
    $rcv_x    =$tmp[ 7]*1.e3 ;
    $rcv_y    =$tmp[ 8]*1.e3 ;
    $rcv_z    =$tmp[ 9]*1.e3 ;
    $trn_x    =$tmp[10]*1.e3 ;
    $trn_y    =$tmp[11]*1.e3 ;
    $trn_z    =$tmp[12]*1.e3 ;
    $trn_bias =$tmp[13]*1.e16;
    $rcv_bias =$tmp[14]*1.e16;
    printf OUT "%5d %14.8e %14.8e %5s %5s %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e\n", $time,$stec,$stec,$trn_id,$rcv_id,$trn_x,$trn_y,$trn_z,$rcv_x,$rcv_y,$rcv_z,$az,$el;
 }
close(IN);
close(OUT);

print "\n$good_count observations accepted and converted.
$bad_count observations rejected as outside 1 day time range.

";
