#!/usr/bin/perl 

# Convert a tpx_vs_gaim cmp file to a fake observation file (obs.in)
# For example, run as: tpx2xyz.pl < gtc001003_cmp_gaim.txt > gtc001003.xyz

use strict;

my $r1 = 6370.e3;
my $r2 = 6370.e3 * 10.;
my $dummy = 0.0e0;
my $DEG2RAD = 3.1415926 / 180.e0;

my ($time, $eflat, $eflon, $trk, $n_pt, $tpx_tec, $tpx_err,
    $gim_tec, $gim_err, $iri_tec, $bent_tec,
    $gim2_tec, $gaim2_tec);
my ($rcvx, $rcvy, $rcvz);
my ($trnx, $trny, $trnz);

my $n = 0;
while ( <> ) {
    ($time, $eflat, $eflon, $trk, $n_pt, $tpx_tec, $tpx_err,
        $gim_tec, $gim_err, $iri_tec, $bent_tec,
        $gim2_tec, $gaim2_tec) = split;
    $n++;

    ($rcvx, $rcvy, $rcvz) = sphcar($eflon*$DEG2RAD, $eflat*$DEG2RAD, $r1);
    ($trnx, $trny, $trnz) = sphcar($eflon*$DEG2RAD, $eflat*$DEG2RAD, $r2);

    printf "%5d %14.8e %14.8e %5s%5d %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e %14.8e\n",
        $time, $tpx_tec*1.e16, $tpx_tec*1.e16,
        $trk, $n_pt, 
        $rcvx, $rcvy, $rcvz, $trnx, $trny, $trnz,
        $dummy, $dummy;
}    


sub sphcar {
    my($lon, $lat, $r) = @_;
    my($x, $y, $z, $x2y2);

    $x2y2 = $r * cos($lat);

    $x = $x2y2 * cos($lon);
    $y = $x2y2 * sin($lon);
    $z = $r * sin($lat);
    return ($x, $y, $z);
}

