#!/usr/bin/perl -w
#######################
# Adapted by Vardan Akopian from original by Lukas Mandrake
#######################

use strict;

#############################################################################
#MAIN BODY
#############################################################################

my ($rslice, $lslice, $help_flag, $image_file, $outfile, $outfile3, $stfile, $xsize, $ysize, $resize, $cmap, $dpi, $format, $matlab);

$matlab = "matlab";

# the location of ImagMagick's convert program
# for unix
my $convert = "convert";
# for windows
# my $convert = "/usr/local/ImageMagick/bin/convert";

if ($#ARGV<0) {
  $help_flag=1;
} else { 
  $help_flag=0;
  $image_file = shift;
  while (@ARGV) {
    my $arg = shift;
    if    ($arg =~ /^-h/     ) { $help_flag=1; last; }
    elsif ($arg =~ /^-rslice/) { $rslice  = shift; }
    elsif ($arg =~ /^-lslice/) { $lslice  = shift; }
    elsif ($arg =~ /^-cmap/  ) { $cmap    = shift; }
    elsif ($arg =~ /^-out/   ) { $outfile = shift; }
    elsif ($arg =~ /^-out3/  ) { $outfile3= shift; }
    elsif ($arg =~ /^-stfile/) { $stfile  = shift; }
    elsif ($arg =~ /^-x/     ) { $xsize   = shift; }
    elsif ($arg =~ /^-y/     ) { $ysize   = shift; }
    elsif ($arg =~ /^-format/) { $format  = shift; }
    elsif ($arg =~ /^-dpi/   ) { $dpi     = shift; }
    else {print "slice: Fatal error, bad command line argument: $arg\n"; $help_flag=1; last;}
  }
}

if (!$image_file || $image_file =~ /^-/ ) {
  $help_flag = 1;
} elsif (! -f $image_file) {
  die "no such file $image_file";
}

if ($help_flag) {
  print <<EOH
Usage: vtec_and_dens_slice <image_file> [switches]

Plots the 3D (pql) voxel grid. Output is generated into any
requesetd format type. The switches are:

-rslice NUMBERS	 The radiuses of slices. Default: "200 300 400 500 600"
-lslice NUMBERS	 The longitudes of slices. Default: "50 140 230 320"
-date   MMDDYY	 The date to be displayed in the title. Default: 000000
-cmap   NAME	 The name of matlab colormap. Default: jet
-format TYPE	 Format of desired output file. Valid responses are TIF,
		 PS, JPG, PNG, PCX. Default: JPG

-out    FILE	 Final tec map image file name. Default: <image_file>\.tec.jpg

-out3    FILE	 Final 3 panel image file name. Default: <image_file>\.3panel.jpg

-stfile FILE	 File name of the satelite location file. Default: none

-x      PIXELS	 Pixel size of desired image. Default: no resize

-y      PIXELS	 Pixel size of desired image. Default: no resize

-dpi    NUMBER	 Dot per inch resolution for the output file. Default: 300

-h         	 Print this help message.

Example: vtec_and_dens_slice kalman_inist_0000_image.mat

EOH
  ;
  exit;
}

if (!$format) { $format = "jpg"; } else { $format = lc($format); }
if (!$outfile) { $outfile = $image_file.".tec.".$format;}
if (!$outfile3) { $outfile3 = $image_file.".3panel.".$format;}
if (!$rslice) { $rslice = "200 300 400 500 600"; }
if (!$lslice) { $lslice = "50 140 230 320"; }
if (!$cmap) { $cmap = "jet"; }
if (!$stfile) { $stfile = ""; }
if (!$xsize && $ysize) { die "resizing requires both x and y sizes"; }
if (!$ysize && $xsize) { die "resizing requires both x and y sizes"; }
if (!$dpi) { $dpi = 300; }


#####YOU CANNOT GENERATE ANYTHING OTHER THAN PS FILES IN MATLAB NODISPLAY
my $workstring =
#  "$matlab -nodesktop -nosplash -r \"addpath $ENV{'GAIM'}/visual/;";
  "$matlab -nodisplay -nojvm -r \"addpath $ENV{'GAIM'}/visual/;";
#  "$matlab -nodisplay -nojvm -r \"";
#  "$matlab -nojvm -r \"";
#$workstring.="overcome_matlab_license('";
$workstring.="eval('";
$workstring.="vtec_and_dens_slice(";
$workstring.="''$image_file'',''$stfile'',[$rslice],[$lslice],''$cmap'',$dpi);";
$workstring.="')";
$workstring.=";   exit;\"";

if (!$xsize && !$ysize) {
  $resize = " ";
} else {
  $resize = " -resize $xsize"."x"."$ysize";
}
#Now we convert the Matlab PS files into whatever we're interested in
$workstring.=" && $convert $resize $image_file.tec.ps $outfile && if [ \"$format\" != \"ps\" ]; then rm -f $image_file.tec.ps; fi; $convert $resize $image_file.3panel.ps $outfile3 && if [ \"$format\" != \"ps\" ]; then rm -f $image_file.3panel.ps; fi\n";
print "Running $workstring...\n\n";
print `$workstring`;
print "Done\n";
